"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificationRequest = exports.MAX_ENERGY_PER_CERTIFICATE = void 0;
const ethers_1 = require("ethers");
const events_1 = require("../utils/events");
const CertificateUtils_1 = require("./CertificateUtils");
// Maximum number Solidity can handle is (2^256)-1
exports.MAX_ENERGY_PER_CERTIFICATE = ethers_1.BigNumber.from(2).pow(256).sub(1);
class CertificationRequest {
    constructor(id, blockchainProperties) {
        this.id = id;
        this.blockchainProperties = blockchainProperties;
        this.initialized = false;
    }
    static async create(generationStartTime, generationEndTime, deviceId, blockchainProperties, forAddress) {
        const newCertificationRequest = new CertificationRequest(null, blockchainProperties);
        const { issuer, activeUser } = blockchainProperties;
        const issuerWithSigner = issuer.connect(activeUser);
        const data = CertificateUtils_1.encodeData({ generationStartTime, generationEndTime, deviceId, metadata: '' });
        const tx = await (forAddress
            ? issuerWithSigner.requestCertificationFor(data, forAddress)
            : issuerWithSigner.requestCertification(data));
        const { events: [certificationRequested] } = await tx.wait();
        const log = issuer.interface.decodeEventLog(certificationRequested.event, certificationRequested.data, certificationRequested.topics);
        newCertificationRequest.id = log._id.toNumber();
        return newCertificationRequest.sync();
    }
    static async getAll(blockchainProperties) {
        const { issuer } = blockchainProperties;
        const certificationRequestedEvents = await events_1.getEventsFromContract(issuer, issuer.filters.CertificationRequested(null, null));
        return certificationRequestedEvents.map((event) => new CertificationRequest(event._id, blockchainProperties));
    }
    async sync() {
        if (this.id === null) {
            return this;
        }
        const { issuer } = this.blockchainProperties;
        const issueRequest = await issuer.getCertificationRequest(this.id);
        const decodedData = CertificateUtils_1.decodeData(issueRequest.data);
        this.owner = issueRequest.owner;
        this.fromTime = decodedData.generationStartTime;
        this.toTime = decodedData.generationEndTime;
        this.deviceId = decodedData.deviceId;
        this.approved = issueRequest.approved;
        this.revoked = issueRequest.revoked;
        const certificationRequestedLogs = await events_1.getEventsFromContract(issuer, issuer.filters.CertificationRequested(null, this.id));
        const certificationApprovedLogs = await events_1.getEventsFromContract(issuer, issuer.filters.CertificationRequestApproved(null, this.id, null));
        this.issuedCertificateTokenId =
            certificationApprovedLogs[0]?._certificateId.toNumber() ?? null;
        const creationBlock = await issuer.provider.getBlock(certificationRequestedLogs[0].blockNumber);
        this.created = Number(creationBlock.timestamp);
        this.initialized = true;
        return this;
    }
    async approve(energy) {
        const { issuer, activeUser } = this.blockchainProperties;
        const validityData = issuer.interface.encodeFunctionData('isRequestValid', [
            this.id.toString()
        ]);
        const issuerWithSigner = issuer.connect(activeUser);
        const approveTx = await issuerWithSigner.approveCertificationRequest(this.id, energy, validityData);
        const { events } = await approveTx.wait();
        const certificateId = Number(events.find((log) => log.event === 'CertificationRequestApproved')
            .topics[3]);
        this.issuedCertificateTokenId = certificateId;
        return certificateId;
    }
    async revoke() {
        const { issuer, activeUser } = this.blockchainProperties;
        const issuerWithSigner = issuer.connect(activeUser);
        const revokeTx = await issuerWithSigner.revokeRequest(this.id);
        await revokeTx.wait();
        return revokeTx;
    }
}
exports.CertificationRequest = CertificationRequest;
//# sourceMappingURL=CertificationRequest.js.map